=begin
Copyright 2012 (c), TIG
All Rights Reserved.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES,INCLUDING,WITHOUT LIMITATION,THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

dcpricer.rb

Usage:
Finds all DCs in a model and reports their hidden==false subcomponent's 
attribute 'prices', if any...
Written specifically for 'thegreek67', but readily customizable.
Type in the Ruby Console: TIG.dcpricer 
OR
Plugins Menu: 'TIG.dcpricer'

Writes "ModelName_DCprices.csv" file with the model and opens it, 
showing sorted/itemised-prices + grand-total.

Donations:
PayPal.com - info @ revitrev.org

Version:
1.0 20120211 First release.
=end
require 'sketchup.rb'
require 'dynamiccomponents.rb'
###
module TIG

unless file_loaded?(File.basename(__FILE__))
  UI.menu('Plugins').add_item('TIG.dcpricer'){self.dcpricer()}
end#unless
file_loaded(File.basename(__FILE__))

def self.dcpricer()
  model=Sketchup.active_model
  ents=model.active_entities
  dcns=[]
  ents.each{|e|
    next unless e.class==Sketchup::ComponentInstance
    dcns << e.definition.name if e.definition.attribute_dictionaries['dynamic_attributes']
  }
  dcns.uniq!
  dcns.sort!
  dcs=[]
  dcns.each{|e|dcs << model.definitions[e]}
  data=[]
  count='1'
  dcs.each{|dc|
    count.next!
    dcname=dc.name
    num=dc.instances.length.to_s
    dc.entities.each{|e|
      next unless e.class==Sketchup::ComponentInstance
      next unless ad=e.attribute_dictionaries['dynamic_attributes']
      next unless e.get_attribute('dynamic_attributes','hidden',nil)==0.0
      subdc=e.get_attribute('dynamic_attributes','_name','??')
      price=e.get_attribute('dynamic_attributes','price','0').gsub(/[^0-9\.]/,'')
      price=sprintf('%.2f', price)
      total='=SUM(C'+count+'*D'+count+')'
      data << dcname+','+subdc+','+num+','+price+','+total
      next
    }
  }
  data=['DCNAME,SUBDC,NUM,PRICE[],TOTAL[]']+data ### insert title
  data << '' << ',,,,'+'=SUM(E2:E'+count+')' ### insert grand-total
  csv=File.join(File.dirname(model.path), model.title+"_DCprices.csv").tr("\\","/") 
  begin ### write to file
    file=File.new(csv,"w")
  rescue ### trap if csv is open
    UI.messagebox("DC Cost Report:\n\n  "+csv+"\n\nCannot be written - it's probably already open.\nClose it and try making it again...\n\nExiting...")
	return nil
  end
  file.puts(data)
  file.close
  UI.openURL('file:///'+csv)
  ###
end

end#module
